import json
import click
import sys
import yaml
from click_option_group import optgroup, RequiredMutuallyExclusiveOptionGroup
from cli.utils import exception_handler, wowza_auth, display_message
from cli.database import Database
from wowpy.resources import get_resource_info
from wowpy.resources import validate_resource, create_resource, delete_resource, update_resource, get_resource_spec
from wowpy.utils import compare_specs, clean_changes
from pygments import highlight
from pygments.lexers import JsonLexer
from pygments.formatters import TerminalTrueColorFormatter

PROFILE_HELP = """
`wowtool` profile, can be set with WOWTOOL_PROFILE env var.
profile settings in ~/.wow/config.json
"""


@click.group()
def resource():
    """
    Resource commands.
    """
    pass


@resource.command(name='create')
@click.option('--spec-file', '-f', required=True, type=click.File(lazy=False),
              help='File that contains your streaming resource specification')
@exception_handler
def create(spec_file):
    """Create streaming resource"""
    specification = yaml.load(spec_file, Loader=yaml.SafeLoader)
    valid = validate_resource(specification)
    if not valid:
        click.echo('The specification template is invalid')
    else:
        click.echo('The specification template is valid')
        
    wowza_auth()
    database = Database()
    
    resource_name = specification['name']
    stored_specification = database.get_specification_by_name(resource_name)
    if stored_specification:
        click.echo('Specification already exist for the resource')
        # TODO: Show changes, Ask to continue
        sys.exit()

    response = create_resource(specification)
    database.insert_specification(response)
    document = {
        'live_stream_name': response['name'],
        'live_stream_id': response['id']
    }
    database.insert_live_stream_mapping(document)
    formatted_json = json.dumps(response, indent=4)
    colorful_json = highlight(formatted_json, JsonLexer(), TerminalTrueColorFormatter(style='solarized-dark'))
    click.echo(colorful_json)


@resource.command(name='delete')
@optgroup.group('Resource identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for resource')
@optgroup.option('--id', help='Resource stream id')
@optgroup.option('--name', help='Resource stream name')
@exception_handler
def delete(id, name):
    """Delete streaming resource"""  
    wowza_auth()
    database = Database()
    if name:   
        if not database.live_streams_table_exist():
            click.echo('In order to use --name parameter please run: wowtool mapper live_streams, first')
            sys.exit()

        id = database.get_live_stream_id_from_name(name)
        if not id:
            click.echo('Live stream not found in current mapping table')
            sys.exit()
    
    stored_specification = database.get_specification_by_id(id)
    if not stored_specification:
        click.echo('Specification does not exist for the resource')
        sys.exit()

    delete_resource(id)
    database.delete_specification(id)
    database.delete_live_stream_mapping(id)
    click.echo('Streaming resource deleted')


@resource.command(name='update')
@click.option('--spec-file', '-f', required=True, type=click.File(lazy=False),
              help='File that contains your streaming resource specification')
@exception_handler
def update(spec_file):
    """Update streaming resource"""  
    specification = yaml.load(spec_file, Loader=yaml.SafeLoader)
    valid = validate_resource(specification)
    if not valid:
        click.echo('The specification template is invalid')
    else:
        click.echo('The specification template is valid')
        
    wowza_auth()
    database = Database()
    
    resource_name = specification['name']
    stored_specification = database.get_specification_by_name(resource_name)
    changes = compare_specs(spec_src=stored_specification, spec_dst=specification)
    changes = clean_changes(changes)
    click.echo('The following changes will be applied:')
    display_message(changes)
    
    # TODO: Ask for apply changes
    specification_with_ids = update_resource(stored_specification, changes)
    click.echo('Updated specification is:')
    display_message(specification_with_ids)
    database.update_specification(specification_with_ids)
    click.echo('Streaming resource succesfully updated')

@resource.command(name='validator')
@click.option('--spec-file', '-f', required=True, type=click.File(lazy=False),
              help='File that contains your streaming resource specification')
@exception_handler
def validator(spec_file):
    """Validate streaming resource specification"""
    specification = yaml.load(spec_file, Loader=yaml.SafeLoader)
    valid = validate_resource(specification)
    if not valid:
        click.echo('The specification template is invalid')
    else:
        click.echo('The specification template is valid')


@resource.command(name='query')
@optgroup.group('Resource identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for resource')
@optgroup.option('--id', help='Resource stream id')
@optgroup.option('--name', help='Resource stream name')
@exception_handler
def query(id, name):
    """Get data for a resource"""
    wowza_auth()
    if name:
        database = Database()
        if not database.live_streams_table_exist():
            click.echo('In order to use --name parameter please run: wowtool mapper live_streams, first')
            sys.exit()

        id = database.get_live_stream_id_from_name(name)
        if not id:
            click.echo('Live stream not found in current mapping table')
            sys.exit()
    response = get_resource_info(id)
    formatted_json = json.dumps(response, indent=4)
    colorful_json = highlight(formatted_json, JsonLexer(), TerminalTrueColorFormatter(style='solarized-dark'))
    click.echo(colorful_json)

@resource.command(name='dump')
@optgroup.group('Resource identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for resource')
@optgroup.option('--id', help='Resource stream id')
@optgroup.option('--name', help='Resource stream name')
@exception_handler
def dump(id, name):
    "Dump resource parameters from wowza"
    wowza_auth()
    if name:
        database = Database()
        if not database.live_streams_table_exist():
            click.echo('In order to use --name parameter please run: wowtool mapper live_streams, first')
            sys.exit()

        id = database.get_live_stream_id_from_name(name)
        if not id:
            click.echo('Live stream not found in current mapping table')
            sys.exit()
    else: # id
        name = database.get_live_stream_name_from_id(id)
        if not name:
            click.echo('Live stream not found in current mapping table')
            sys.exit()
    response = get_resource_spec(id)
    # TODO: avoid to save twice and existing specification
    database.insert_specification(response)
    with open('templates/'+name+'.yaml', 'w') as outfile:
        yaml.dump(response, outfile, default_flow_style=False, sort_keys=False)

    formatted_json = json.dumps(response, indent=4)
    colorful_json = highlight(formatted_json, JsonLexer(), TerminalTrueColorFormatter(style='solarized-dark'))
    click.echo(colorful_json)



from cli.utils import get_base_file_path
from cli.constants import DATABASE_FILENAME
from tinydb import TinyDB, Query

class Database:

  def __init__(self, database_file_path=None):
    if not database_file_path:
      database_file_path = self.get_database_file_path()
    self.db = TinyDB(database_file_path)

  def get_database_file_path(self):
    file_path = get_base_file_path(DATABASE_FILENAME)
    return file_path

  def get_recording_ids_from_name(self, name):
    recording_ids = []
    table_name = 'recordings'
    query = Query()
    table = self.db.table(table_name)
    response = table.search(query.live_stream_name == name)
    if response:
      for item in response:
        recording_ids.append(item.get('recording_id', None))
    return recording_ids

  def get_live_stream_id_from_name(self, name):
    id = None
    table_name = 'live_streams'
    query = Query()
    table = self.db.table(table_name)
    response = table.search(query.live_stream_name == name)
    if response:
      id = response[0].get('live_stream_id', None)
    return id

  def get_live_stream_name_from_id(self, id):
    name = ''
    table_name = 'live_streams'
    query = Query()
    table = self.db.table(table_name)
    response = table.search(query.live_stream_id == id)
    if response:
      name = response[0].get('live_stream_name', None)
    return name

  def recordings_table_exist(self):
    table_name = 'recordings'
    tables = self.db.tables()
    exist = table_name in tables
    return exist

  def live_streams_table_exist(self):
    table_name = 'live_streams'
    tables = self.db.tables()
    exist = table_name in tables
    return exist

  def insert_live_stream_mappings(self, document_batch):
    table_name = 'live_streams'
    self.db.drop_table(table_name)
    table = self.db.table(table_name)
    table.insert_multiple(document_batch)

  def insert_live_stream_mapping(self, document):
    table_name = 'live_streams'
    table = self.db.table(table_name)
    table.insert(document)

  def delete_live_stream_mapping(self, id):
    Spec = Query()
    table_name = 'live_streams'
    table = self.db.table(table_name)
    table.remove(Spec.live_stream_id.matches(id))

  def insert_recording_mappings(self, document_batch):
    table_name = 'recordings'
    self.db.drop_table(table_name)
    table = self.db.table(table_name)
    table.insert_multiple(document_batch)

  def insert_specification(self, specification):
    table_name = 'specifications'
    table = self.db.table(table_name)
    table.insert(specification)

  def delete_specification(self, id):
    Spec = Query()
    table_name = 'specifications'
    table = self.db.table(table_name)
    table.remove(Spec.id.matches(id))

  def update_specification(self, specification):
    Spec = Query()
    table_name = 'specifications'
    table = self.db.table(table_name)
    id = specification['id']
    table.update(specification, Spec.id.matches(id))

  def get_specification_by_name(self, name):
    Spec = Query()
    table_name = 'specifications'
    table = self.db.table(table_name)
    stored_specification = table.search(Spec.name.matches(name))

    if not stored_specification:
      return None
    else:
      return stored_specification[0] # convert list to single item

  def get_specification_by_id(self, id):
    Spec = Query()
    table_name = 'specifications'
    table = self.db.table(table_name)
    stored_specification = table.search(Spec.id.matches(id))
    return stored_specification



import pytest
import json
from cli.database import Database

@pytest.fixture
def database():
    database = Database(database_file_path='tests/fixtures/test_database.json')
    return database

@pytest.fixture
def spec_one_output_template():
    with open('tests/fixtures/spec_one_output_template.json') as json_file:
        specification = json.load(json_file)
    return specification

@pytest.fixture
def spec_one_output_database():
    with open('tests/fixtures/spec_one_output_database.json') as json_file:
        specification = json.load(json_file)
    return specification

def test_live_streams_table_exist(database):
    exist = database.live_streams_table_exist()
    assert exist in [False, True]

def test_insert_specification(database, spec_one_output_database):
    database.insert_specification(spec_one_output_database)
    assert 1==1

def test_insert_live_stream_mapping(database):
    document = {
        'live_stream_name': 'Baton-Test',
        'live_stream_id': 'mbwlljdc'
    }
    database.insert_live_stream_mapping(document)
    assert 1==1

def test_get_live_stream_id_from_name(database):
    id = database.get_live_stream_id_from_name('Baton-Test')
    assert id=='mbwlljdc'

def test_delete_specification(database, spec_one_output_database):
    id = spec_one_output_database['id']
    database.delete_specification(id)
    assert 1==1

def test_delete_live_stream_mapping(database, spec_one_output_database):
    id = spec_one_output_database['id']
    database.delete_live_stream_mapping(id)
    assert 1==1